@@ -1,4 +1,4 @@
-/* $Id: pg_wchar.h,v 1.37 2001/11/05 17:46:33 momjian Exp $ */
+/* $Id: pg_wchar.h,v 1.44.2.1 2003/02/19 14:14:58 ishii Exp $ */
 
 #ifndef PG_WCHAR_H
 #define PG_WCHAR_H
@@ -15,13 +15,8 @@
 /*
  * The pg_wchar
  */
-#ifdef MULTIBYTE
 typedef unsigned int pg_wchar;
 
-#else
-#define pg_wchar char
-#endif
-
 /*
  * various definitions for EUC
  */
@@ -43,11 +38,11 @@ typedef unsigned int pg_wchar;
  */
 #define IS_LCPRV1(c)	((unsigned char)(c) == 0x9a || (unsigned char)(c) == 0x9b)
 /*
- * Is a leading byte for "official" multi byte encodings?
+ * Is a leading byte for "official" multibyte encodings?
  */
 #define IS_LC2(c)	((unsigned char)(c) >= 0x90 && (unsigned char)(c) <= 0x99)
 /*
- * Is a prefix byte for "private" multi byte encodings?
+ * Is a prefix byte for "private" multibyte encodings?
  */
 #define IS_LCPRV2(c)	((unsigned char)(c) == 0x9c || (unsigned char)(c) == 0x9d)
 
@@ -123,7 +118,7 @@ typedef unsigned int pg_wchar;
 #define LC_ARABIC_2_COLUMN	0xa8	/* Arabic 1-column (not supported) */
 
 /*
- * Private multi byte encodings (0xf0-0xff)
+ * Private multibyte encodings (0xf0-0xff)
  */
 #define LC_INDIAN_1_COLUMN	0xf0/* Indian charset for 1-column width
 								 * glypps (not supported) */
@@ -145,7 +140,7 @@ typedef unsigned int pg_wchar;
  * Encoding numeral identificators
  *
  * WARNING: the order of this table must be same as order
- *			in the pg_enconv[] (mb/conv.c) and pg_enc2name[] (mb/encnames.c) array!
+ *			in the pg_enc2name[] (mb/encnames.c) array!
  *
  *			If you add some encoding don'y forget check
  *			PG_ENCODING_[BE|FE]_LAST macros.
@@ -159,6 +154,7 @@ typedef enum pg_enc
 	PG_EUC_CN,					/* EUC for Chinese */
 	PG_EUC_KR,					/* EUC for Korean */
 	PG_EUC_TW,					/* EUC for Taiwan */
+	PG_JOHAB,					/* EUC for Korean JOHAB */
 	PG_UTF8,					/* Unicode UTF-8 */
 	PG_MULE_INTERNAL,			/* Mule internal code */
 	PG_LATIN1,					/* ISO-8859-1 Latin 1 */
@@ -171,6 +167,9 @@ typedef enum pg_enc
 	PG_LATIN8,					/* ISO-8859-14 Latin8 */
 	PG_LATIN9,					/* ISO-8859-15 Latin9 */
 	PG_LATIN10,					/* ISO-8859-16 Latin10 */
+	PG_WIN1256,					/* windows-1256 */
+	PG_TCVN,					/* TCVN (Windows-1258) */
+	PG_WIN874,					/* windows-874 */
 	PG_KOI8R,					/* KOI8-R */
 	PG_WIN1251,					/* windows-1251 (was: WIN) */
 	PG_ALT,						/* (MS-DOS CP866) */
@@ -180,19 +179,18 @@ typedef enum pg_enc
 	PG_ISO_8859_8,				/* ISO-8859-8 */
 
 	/* followings are for client encoding only */
-	PG_SJIS,					/* Shift JIS */
-	PG_BIG5,					/* Big5 */
+	PG_SJIS,					/* Shift JIS (Winindows-932) */
+	PG_BIG5,					/* Big5 (Windows-950) */
+	PG_GBK,						/* GBK (Windows-936) */
+	PG_UHC,						/* UHC (Windows-949) */
 	PG_WIN1250,					/* windows-1250 */
-
+	PG_GB18030,					/* GB18030 */
 	_PG_LAST_ENCODING_			/* mark only */
 
 } pg_enc;
 
 #define PG_ENCODING_BE_LAST PG_ISO_8859_8
-#define PG_ENCODING_FE_LAST PG_WIN1250
-
-
-#ifdef MULTIBYTE
+#define PG_ENCODING_FE_LAST PG_WIN1256
 
 /*
  * Please use these tests before access to pg_encconv_tbl[]
@@ -242,30 +240,6 @@ extern pg_encname *pg_char_to_encname_struct(const char *name);
 extern int	pg_char_to_encoding(const char *s);
 extern const char *pg_encoding_to_char(int encoding);
 
-typedef void (*to_mic_converter) (unsigned char *l, unsigned char *p, int len);
-typedef void (*from_mic_converter) (unsigned char *mic, unsigned char *p, int len);
-
-/*
- * The backend encoding conversion routines
- * Careful:
- *
- *	if (PG_VALID_ENCODING(enc))
- *		pg_encconv_tbl[ enc ]->foo
- */
-#ifndef FRONTEND
-typedef struct pg_enconv
-{
-	pg_enc		encoding;		/* encoding identifier */
-	to_mic_converter to_mic;	/* client encoding to MIC */
-	from_mic_converter from_mic;	/* MIC to client encoding */
-	to_mic_converter to_unicode;	/* client encoding to UTF-8 */
-	from_mic_converter from_unicode;	/* UTF-8 to client encoding */
-} pg_enconv;
-
-extern pg_enconv pg_enconv_tbl[];
-extern pg_enconv *pg_get_enconv_by_encoding(int encoding);
-#endif   /* FRONTEND */
-
 /*
  * pg_wchar stuff
  */
@@ -276,9 +250,9 @@ typedef int (*mblen_converter) (const unsigned char *mbstr);
 
 typedef struct
 {
-	mb2wchar_with_len_converter mb2wchar_with_len;		/* convert a multi-byte
+	mb2wchar_with_len_converter mb2wchar_with_len;		/* convert a multibyte
 														 * string to a wchar */
-	mblen_converter mblen;		/* returns the length of a multi-byte char */
+	mblen_converter mblen;		/* returns the length of a multibyte char */
 	int			maxmblen;		/* max bytes for a char in this charset */
 } pg_wchar_tbl;
 
@@ -302,28 +276,30 @@ typedef struct
 	unsigned int utf;			/* UTF-8 */
 } pg_local_to_utf;
 
-extern int	pg_mb2wchar(const unsigned char *, pg_wchar *);
-extern int	pg_mb2wchar_with_len(const unsigned char *, pg_wchar *, int);
-extern int	pg_char_and_wchar_strcmp(const char *, const pg_wchar *);
-extern int	pg_wchar_strncmp(const pg_wchar *, const pg_wchar *, size_t);
-extern int	pg_char_and_wchar_strncmp(const char *, const pg_wchar *, size_t);
-extern size_t pg_wchar_strlen(const pg_wchar *);
-extern int	pg_mblen(const unsigned char *);
-extern int	pg_encoding_mblen(int, const unsigned char *);
-extern int	pg_mule_mblen(const unsigned char *);
-extern int	pg_mic_mblen(const unsigned char *);
-extern int	pg_mbstrlen(const unsigned char *);
-extern int	pg_mbstrlen_with_len(const unsigned char *, int);
-extern int	pg_mbcliplen(const unsigned char *, int, int);
-extern int	pg_mbcharcliplen(const unsigned char *, int, int);
-extern int	pg_encoding_max_length(int);
+extern int	pg_mb2wchar(const unsigned char *from, pg_wchar *to);
+extern int	pg_mb2wchar_with_len(const unsigned char *from, pg_wchar *to, int len);
+extern int	pg_char_and_wchar_strcmp(const char *s1, const pg_wchar *s2);
+extern int	pg_wchar_strncmp(const pg_wchar *s1, const pg_wchar *s2, size_t n);
+extern int	pg_char_and_wchar_strncmp(const char *s1, const pg_wchar *s2, size_t n);
+extern size_t pg_wchar_strlen(const pg_wchar *wstr);
+extern int	pg_mblen(const unsigned char *mbstr);
+extern int	pg_encoding_mblen(int encoding, const unsigned char *mbstr);
+extern int	pg_mule_mblen(const unsigned char *mbstr);
+extern int	pg_mic_mblen(const unsigned char *mbstr);
+extern int	pg_mbstrlen(const unsigned char *mbstr);
+extern int	pg_mbstrlen_with_len(const unsigned char *mbstr, int len);
+extern int	pg_mbcliplen(const unsigned char *mbstr, int len, int limit);
+extern int	pg_mbcharcliplen(const unsigned char *mbstr, int len, int imit);
+extern int	pg_encoding_max_length(int encoding);
 extern int	pg_database_encoding_max_length(void);
 
-extern int	pg_set_client_encoding(int);
+extern void SetDefaultClientEncoding(void);
+extern int	SetClientEncoding(int encoding, bool doit);
+extern void	InitializeClientEncoding(void);
 extern int	pg_get_client_encoding(void);
 extern const char *pg_get_client_encoding_name(void);
 
-extern void SetDatabaseEncoding(int);
+extern void SetDatabaseEncoding(int encoding);
 extern int	GetDatabaseEncoding(void);
 extern const char *GetDatabaseEncodingName(void);
 
@@ -331,20 +307,30 @@ extern int	pg_valid_client_encoding(const char *name);
 extern int	pg_valid_server_encoding(const char *name);
 
 extern int	pg_utf_mblen(const unsigned char *);
-extern int pg_find_encoding_converters(int src, int dest,
-							to_mic_converter *src_to_mic,
-							from_mic_converter *dest_from_mic);
 extern unsigned char *pg_do_encoding_conversion(unsigned char *src, int len,
-						  to_mic_converter src_to_mic,
-						  from_mic_converter dest_from_mic);
+						  int src_encoding,
+						  int dest_encoding);
 
-extern unsigned char *pg_client_to_server(unsigned char *, int);
-extern unsigned char *pg_server_to_client(unsigned char *, int);
+extern unsigned char *pg_client_to_server(unsigned char *s, int len);
+extern unsigned char *pg_server_to_client(unsigned char *s, int len);
 
-extern unsigned short BIG5toCNS(unsigned short, unsigned char *);
-extern unsigned short CNStoBIG5(unsigned short, unsigned char);
+extern unsigned short BIG5toCNS(unsigned short big5, unsigned char *lc);
+extern unsigned short CNStoBIG5(unsigned short cns, unsigned char lc);
 
-char	   *pg_verifymbstr(const unsigned char *, int);
-#endif   /* MULTIBYTE */
+extern void LocalToUtf(unsigned char *iso, unsigned char *utf,
+		   pg_local_to_utf *map, int size, int encoding, int len);
+
+extern void UtfToLocal(unsigned char *utf, unsigned char *iso,
+		   pg_utf_to_local *map, int size, int len);
+
+extern char *pg_verifymbstr(const unsigned char *mbstr, int len);
+
+extern void pg_ascii2mic(unsigned char *src, unsigned char *dest, int len);
+extern void pg_mic2ascii(unsigned char *src, unsigned char *dest, int len);
+extern void pg_print_bogus_char(unsigned char **mic, unsigned char **p);
+extern void latin2mic(unsigned char *l, unsigned char *p, int len, int lc);
+extern void mic2latin(unsigned char *mic, unsigned char *p, int len, int lc);
+extern void latin2mic_with_table(unsigned char *l, unsigned char *p, int len, int lc, unsigned char *tab);
+extern void mic2latin_with_table(unsigned char *mic, unsigned char *p, int len, int lc, unsigned char *tab);
 
 #endif   /* PG_WCHAR_H */
